@@ -70,10 +70,12 @@ colortyp colvar;
 #define false 0
 char *eval_str();
 void var_getstr(int varnum,char *s);
-int pass_marker(char *s);
+//int pass_marker(char *s);
 int f_eof(int chn);
 void gle_as_a_calculator(vector<string>* exprs);
 void output_error_cerr(ParserError& err);
+double xg3d(double x, double y, double z);
+double yg3d(double x, double y, double z);
 
 /*---------------------------------------------------------------------------*/
 /* bin = 10..29, binstr = 30..49, fn= 60...139, userfn=LOCAL_START_INDEX..nnn */
@@ -159,7 +161,7 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 	if (plen > 1000) {
 		gprint("Expression is suspiciously long %d \n",plen);
 	}
-	union {double d; int l[1];} both;
+	union {double d; int l[2];} both;
 	char *ss2, *ss;
 	double x1, y1, x2, y2;
 	double xx, yy;
@@ -738,13 +740,10 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 			stk[++nstk] = yy;
 			break;
 		case 105: /* CVTGRAY(.5) */
-			colvar.b[B_F] = 1;
-			colvar.b[B_R] = (unsigned char) floor(stk[nstk]*255);
-			colvar.b[B_G] = colvar.b[B_R];
-			colvar.b[B_B] = colvar.b[B_R];
-			both.l[0] = colvar.l;
-			both.l[1] = 0;
-			memcpy(&stk[nstk],&both.d,sizeof(double));
+			{
+				GLERC<GLEColor> color(new GLEColor(stk[nstk]));
+				stk[nstk] = color->getDoubleEncoding();
+			}
 			break;
 		case 106: /* CVTINT(2) */
 			*otyp = 1;
@@ -768,33 +767,48 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 			both.l[1] = 0;
 			memcpy(&stk[nstk],&both.d,sizeof(double));
 			break;
+		case 60+FN_JUSTIFY: /* JUSTIFY(m$) */
+			*otyp = 1;
+			both.l[0] = pass_justify(stk_str[nstk]);
+			both.l[1] = 0;
+			memcpy(&stk[nstk], &both.d, sizeof(double));
+			break;         
 		case 109: /* CVTCOLOR(c$) */
 			*otyp = 1;
-			both.l[0] = pass_color(stk_str[nstk]);
-			both.l[1] = 0;
-			memcpy(&stk[nstk],&both.d,sizeof(double));
+			{
+				GLERC<GLEColor> color(pass_color_var(stk_str[nstk]));
+				stk[nstk] = color->getDoubleEncoding();
+			}
 			break;
 		case 107: /* RGB(.4,.4,.2) */
-			colvar.b[B_F] = 1;
-			colvar.b[B_B] = float_to_color_comp(stk[nstk]);
-			colvar.b[B_G] = float_to_color_comp(stk[nstk-1]);
-			colvar.b[B_R] = float_to_color_comp(stk[nstk-2]);
-			nstk -= 2;
-			both.l[0] = colvar.l;
-			both.l[1] = 0;
-			memcpy(&both.l[0],&colvar.l,sizeof(int));
-			memcpy(&stk[nstk],&both.d,sizeof(double));
+			{
+				GLERC<GLEColor> color(new GLEColor(stk[nstk-2], stk[nstk-1], stk[nstk]));
+				nstk -= 2;
+				stk[nstk] = color->getDoubleEncoding();
+			}
 			break;
 		case 140: /* RGB255(.4,.4,.2) */
-			colvar.b[B_F] = 1;
-			colvar.b[B_B] = float_to_color_comp_255(stk[nstk]);
-			colvar.b[B_G] = float_to_color_comp_255(stk[nstk-1]);
-			colvar.b[B_R] = float_to_color_comp_255(stk[nstk-2]);
-			nstk -= 2;
-			both.l[0] = colvar.l;
-			both.l[1] = 0;
-			memcpy(&both.l[0],&colvar.l,sizeof(int));
-			memcpy(&stk[nstk],&both.d,sizeof(double));
+			{
+				GLERC<GLEColor> color(new GLEColor());
+				color->setRGB255(stk[nstk-2], stk[nstk-1], stk[nstk]);
+				nstk -= 2;
+				stk[nstk] = color->getDoubleEncoding();
+			}
+			break;
+		case 60+FN_RGBA:
+			{
+				GLERC<GLEColor> color(new GLEColor(stk[nstk-3], stk[nstk-2], stk[nstk-1], stk[nstk]));
+				nstk -= 3;
+				stk[nstk] = color->getDoubleEncoding();
+			}
+			break;
+		case 60+FN_RGBA255:
+			{
+				GLERC<GLEColor> color(new GLEColor());
+				color->setRGBA255(stk[nstk-3], stk[nstk-2], stk[nstk-1], stk[nstk]);
+				nstk -= 3;
+				stk[nstk] = color->getDoubleEncoding();
+			}
 			break;
 		case 60+FN_NDATA: /* Number of datapoints in a dateset */
 			*otyp = 1;
@@ -811,10 +825,14 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 			if (i <= 0 || i >= MAX_NB_DATA || dp[i] == NULL)
 				throw g_format_parser_error("dataset d%d not defined", i);
 			else
-				if (j <= 0 || j > dp[i]->np)
+				if (j <= 0 || j > (int)dp[i]->np) {
 					throw g_format_parser_error("index out of range: %d (1 ... %d)", j, dp[i]->np);
-				else
-					stk[nstk] = dp[i]->xv[j-1];
+				} else {
+					GLEArrayImpl* array = dp[i]->getDimData(0);
+					if (array != 0) {
+						stk[nstk] = array->getDouble(j-1);
+					}
+				}
 			break;
 		case 60+FN_DATAYVALUE: /* Y value in a dateset */
 			*otyp = 1; nstk -= 1;
@@ -823,10 +841,22 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 			if (i <= 0 || i >= MAX_NB_DATA || dp[i] == NULL)
 				throw g_format_parser_error("dataset d%d not defined", i);
 			else
-				if (j <= 0 || j > dp[i]->np)
+				if (j <= 0 || j > (int)dp[i]->np) {
 					throw g_format_parser_error("index out of range: %d (1 ... %d)", j, dp[i]->np);
-				else
-					stk[nstk] = dp[i]->yv[j-1];
+				} else {
+					GLEArrayImpl* array = dp[i]->getDimData(1);
+					if (array != 0) {
+						stk[nstk] = array->getDouble(j-1);
+					}
+				}
+			break;
+		case 60+FN_XG3D:
+			stk[nstk - 2] = xg3d(stk[nstk - 2], stk[nstk - 1], stk[nstk]);
+            nstk -= 2;
+			break;
+		case 60+FN_YG3D:
+			stk[nstk - 2] = yg3d(stk[nstk - 2], stk[nstk - 1], stk[nstk]);
+            nstk -= 2;
 			break;
 		default:  /* User function, LOCAL_START_INDEX..nnn , or error */
 			/* Is it a user defined function */
@@ -846,7 +876,8 @@ void eval_pcode_loop(int *pcode, int plen, int *otyp) throw(ParserError) {
 
 GLESub* eval_subroutine_call(int *pcode, int *cp, int* otyp) throw(ParserError) {
 	if (*(pcode+(*cp)++) != 1) {
-		gprint("PCODE, Expecting expression, v=%ld cp=%d \n",*(pcode+(--*(cp))),*cp);
+		(*cp)--;
+		gprint("PCODE, Expecting expression, v=%ld cp=%d \n", *(pcode + (*cp)), *cp);
 		return NULL;
 	}
 	int plen = pcode[(*cp)++];
@@ -913,12 +944,12 @@ void eval(int *pcode, int *cp, double *oval, const char **ostr, int *otyp) throw
 		both.l[1] = 0;
 		dbg gprint("Constant %ld \n",both.l[0]);
 		memcpy(oval,&both.d,sizeof(both.d));
-		memcpy(&both.d,oval,sizeof(both.d));
 		++(*cp);
 		return;
 	}
 	if (pcode[(*cp)++] != 1) {
-		gprint("PCODE, Expecting expression, v=%ld cp=%d \n",*(pcode+(--*(cp))),*cp);
+		(*cp)--;
+		gprint("PCODE, Expecting expression, v=%ld cp=%d \n", *(pcode + (*cp)), *cp);
 		return;
 	}
 	int plen = pcode[(*cp)++];
