@@ -78,6 +78,23 @@ void gle_memory_cell_print(GLEMemoryCell* a, ostream& out) {
 	}
 }
 
+bool gle_memory_cell_to_double(GLEMemoryCell* a, double* result) {
+	switch (a->Type) {
+		case GLE_MC_BOOL:
+			*result = a->Entry.BoolVal ? 1.0 : 0.0;
+			return true;
+		case GLE_MC_INT:
+			*result = a->Entry.IntVal;
+			return true;
+		case GLE_MC_DOUBLE:
+			*result = a->Entry.DoubleVal;
+			return true;
+		default:
+			*result = 0.0;
+			return false;
+	}
+}
+
 GLEDataObject::GLEDataObject() {
 }
 
@@ -95,6 +112,28 @@ bool GLEDataObject::equals(GLEDataObject* obj) {
 void GLEDataObject::print(ostream& out) const {
 }
 
+GLEPointDataObject::GLEPointDataObject(double x, double y) : m_point(x, y) {
+}
+
+GLEPointDataObject::~GLEPointDataObject() {
+}
+
+int GLEPointDataObject::getType() {
+	return GLEObjectTypePoint;
+}
+
+bool GLEPointDataObject::equals(GLEDataObject* obj) {
+	if (obj->getType() != GLEObjectTypePoint) return false;
+	GLEPointDataObject* other = (GLEPointDataObject*)obj;
+	return m_point.getX() == other->m_point.getX()
+		   && m_point.getY() == other->m_point.getY();
+}
+
+void GLEPointDataObject::print(ostream& out) const {
+	m_point.write(out);
+}
+
+
 GLEString::GLEString() {
 	m_Data = NULL;
 	m_Length = 0;
@@ -146,6 +185,45 @@ void GLEString::fromUTF8(const string& str) {
 	0001 0000-0010 FFFF | 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
 */
 
+unsigned int getUTF8NumberOfChars(const char* str, unsigned int len) {
+	unsigned int i = 0;
+	unsigned int dest = 0;
+	while (i < len) {
+		unsigned char ch = str[i++];
+		if ((ch & 0x80) != 0) {
+			/* highest bit is one - 3 possibilities */
+			int remain = 0;
+			if ((ch & 0xE0) == 0xC0) {
+				/* 110x -> two byte unicode */
+				remain = 1;
+			} else if ((ch & 0xF0) == 0xE0) {
+				/* 1110x -> three byte unicode */
+				remain = 2;
+			} else if ((ch & 0xF8) == 0xF0) {
+				/* 1111x -> four byte unicode */
+				remain = 3;
+			} else if ((ch & 0xFC) == 0xF8) {
+				remain = 4;
+			} else if ((ch & 0xFE) == 0xFC) {
+				remain = 5;
+			}
+			while ((remain > 0) && (i < len)) {
+				remain--;
+				ch = str[i++];
+				if ((ch & 0xC0) != 0x80) {
+					remain = 0;
+					i--;
+					/* try processing this byte separately */
+				}
+			}
+			dest++;
+		} else {
+			dest++;
+		}
+	}
+	return dest;
+}
+
 void GLEString::fromUTF8(const char* str, unsigned int len) {
 	resize(len);
 	unsigned int i = 0;
@@ -469,12 +547,19 @@ GLEArrayImpl::GLEArrayImpl() {
 }
 
 GLEArrayImpl::~GLEArrayImpl() {
+	clear();
+}
+
+void GLEArrayImpl::clear() {
 	if (m_Data != NULL) {
 		for (unsigned int i = 0; i < m_Length; i++) {
 			GLE_MC_DEL_INTERN(&m_Data[i]);
 		}
 		free(m_Data);
 	}
+	m_Data = NULL;
+	m_Length = 0;
+	m_Alloc = 0;
 }
 
 int GLEArrayImpl::getType(unsigned int i) {
@@ -503,6 +588,12 @@ void GLEArrayImpl::addObject(GLEDataObject* v) {
 	GLE_MC_SET_OBJECT(&m_Data[len], v);
 }
 
+void GLEArrayImpl::addInt(int v) {
+	unsigned int len = m_Length;
+	ensure(len+1);
+	GLE_MC_SET_INT(&m_Data[len], v);
+}
+
 void GLEArrayImpl::set(unsigned int i, const GLEMemoryCell* cell) {
 	if (cell->Type == GLE_MC_OBJECT) {
 		setObject(i, cell->Entry.ObjectVal);
@@ -540,11 +631,32 @@ void GLEArrayImpl::setBool(unsigned int i, bool v) {
 	GLE_MC_SET_BOOL(&m_Data[i], v);
 }
 
+void GLEArrayImpl::setUnknown(unsigned int i) {
+	GLE_MC_SET_UNKNOWN(&m_Data[i]);
+}
+
+bool GLEArrayImpl::isUnknown(unsigned int i) {
+	return m_Data[i].Type == GLE_MC_UNKNOWN;
+}
+
 GLEDataObject* GLEArrayImpl::getObject(unsigned int i) {
 	if (m_Data[i].Type == GLE_MC_OBJECT) return m_Data[i].Entry.ObjectVal;
 	else return NULL;
 }
 
+GLERC<GLEString> GLEArrayImpl::getString(unsigned int i) {
+	GLERC<GLEString> result;
+	GLEMemoryCell* cell = &m_Data[i];
+	if (cell->Type == GLE_MC_OBJECT && cell->Entry.ObjectVal->getType() == GLEObjectTypeString) {
+		result = static_cast<GLEString*>(cell->Entry.ObjectVal);
+	} else {
+		ostringstream out;
+		gle_memory_cell_print(cell, out);
+		result = new GLEString(out.str());
+	}
+	return result;
+}
+
 void GLEArrayImpl::ensure(unsigned int size) {
 	if (m_Alloc < size) extend(size);
 	while (m_Length < size) {
@@ -554,6 +666,16 @@ void GLEArrayImpl::ensure(unsigned int size) {
 }
 
 void GLEArrayImpl::resize(unsigned int size) {
+	ensure(size);
+	if (size < m_Length) {
+		for (unsigned int i = size; i < m_Length; i++) {
+			init(i);
+		}
+		m_Length = size;
+	}
+}
+
+void GLEArrayImpl::resizeMemory(unsigned int size) {
 	if (m_Alloc < size) {
 		m_Data = (GLEMemoryCell*)realloc(m_Data, size*sizeof(GLEMemoryCell));
 		m_Alloc = size;
@@ -565,7 +687,7 @@ void GLEArrayImpl::extend(unsigned int size) {
 	while (newSize < size) {
 		newSize = 2*newSize + 5;
 	}
-	resize(newSize);
+	resizeMemory(newSize);
 }
 
 void GLEArrayImpl::enumStrings(ostream& out) {
@@ -631,7 +753,7 @@ GLEDynamicSub::GLEDynamicSub(GLESub* sub) {
 
 GLEDynamicSub::~GLEDynamicSub() {
 	if (m_VarValues != NULL) delete m_VarValues;
-	if (m_State != NULL) free(m_State);
+	if (m_State != NULL) delete m_State;
 }
 
 int GLEDynamicSub::getType() {
@@ -713,3 +835,53 @@ void GLEObjectRepresention::copyChildrenRecursive(GLEObjectRepresention* newobj,
 		}
 	}
 }
+
+GLEClassDefinition::GLEClassDefinition(const char* name):
+	m_Name(new GLEString(name)),
+	m_FieldNames(new GLEArrayImpl())
+{
+}
+
+void GLEClassDefinition::addField(const char* fieldName) {
+	m_FieldNames->addObject(new GLEString(fieldName));
+}
+
+int GLEClassDefinition::getType() {
+	return GLEObjectTypeClassDefinition;
+}
+
+bool GLEClassDefinition::equals(GLEDataObject* obj) {
+	return false;
+}
+
+void GLEClassDefinition::print(ostream& out) const {
+}
+
+GLEClassInstance::GLEClassInstance(GLEClassDefinition* definition):
+	m_Definition(definition)
+{
+}
+
+int GLEClassInstance::getType() {
+	return GLEObjectTypeClassInstance;
+}
+
+bool GLEClassInstance::equals(GLEDataObject* obj) {
+	return false;
+}
+
+void GLEClassInstance::print(ostream& out) const {
+}
+
+GLEClassInstance* getGLEClassInstance(GLEMemoryCell* object, GLEClassDefinition* def) {
+	if (object->Type == GLE_MC_OBJECT) {
+		GLEDataObject* dataObject = object->Entry.ObjectVal;
+		if (dataObject->getType() == GLEObjectTypeClassInstance) {
+			GLEClassInstance* classInstance = static_cast<GLEClassInstance*>(dataObject);
+			if (classInstance->getDefinition() == def) {
+				return classInstance;
+			}
+		}
+	}
+	return 0;
+}
