@@ -60,6 +60,7 @@
 #include "pass.h"
 #include "keyword.h"
 #include "run.h"
+#include "gle-poppler.h"
 
 // TODO make these available through the interface
 extern CmdLineObj g_CmdLine;
@@ -141,12 +142,6 @@ GLEInterface::GLEInterface() {
 	// Initialize text property store model
 	m_TextModel = new GLEPropertyStoreModel();
 	m_TextModel->add(new GLEPropertyFont("Font"));
-	GLEPropertyNominal* fontstyle = new GLEPropertyNominal("Font style", GLEPropertyTypeInt, GLEDOPropertyFontStyle);
-	fontstyle->addValue("roman", GLEFontStyleRoman);
-	fontstyle->addValue("bold", GLEFontStyleBold);
-	fontstyle->addValue("italic", GLEFontStyleItalic);
-	fontstyle->addValue("bold+italic", GLEFontStyleBoldItalic);
-	m_TextModel->add(fontstyle);
 	m_TextModel->add(new GLEPropertyHei("Font size"));
 	m_TextModel->add(new GLEPropertyColor("Text color"));
 	m_TextModel->add(new GLEPropertyJustify("Text justify"));
@@ -191,8 +186,9 @@ GLEInterface::~GLEInterface() {
 	if (m_FileInfoMap != NULL) delete m_FileInfoMap;
 }
 
-bool GLEInterface::initializeGLE(const char* appname, char **argv) {
+bool GLEInterface::initializeGLE(const char* appname, int argc, char **argv) {
 	try {
+      gle_glib_init(argc, argv);
 		// Initialize code in core.cpp
 		g_init();
 		// Load configuration file
@@ -427,10 +423,15 @@ GLEColor* GLEInterface::getColor(int i) {
 }
 
 void GLEInterface::addFont(GLEFont* font) {
-	m_FontHash->add_item(font->getName(), m_Fonts.size());
-	m_FontIndexHash->add_item(font->getIndex(), m_Fonts.size());
 	font->setNumber(m_Fonts.size());
 	m_Fonts.add(font);
+	addSubFont(font);
+}
+
+void GLEInterface::addSubFont(GLEFont* font) {
+	m_FontHash->add_item(font->getName(), m_AllFonts.size());
+	m_FontIndexHash->add_item(font->getIndex(), m_AllFonts.size());
+	m_AllFonts.add(font);
 }
 
 // Return the number of available fonts
@@ -448,15 +449,20 @@ GLEFont* GLEInterface::getFont(int i) {
 GLEFont* GLEInterface::getFont(const string& name) {
 	int idx = m_FontHash->try_get(name);
 	if (idx == -1) return NULL;
-	else return m_Fonts.get(idx);
+	else return m_AllFonts.get(idx);
 }
 
 GLEFont* GLEInterface::getFont(const char* name) {
 	return getFont(string(name));
 }
 
-int GLEInterface::getFontIndex(int font) {
-	return m_FontIndexHash->try_get(font);
+GLEFont* GLEInterface::getFontIndex(int font) {
+	int index = m_FontIndexHash->try_get(font);
+	if (index == -1) {
+		return 0;
+	} else {
+		return m_AllFonts.get(index);
+	}
 }
 
 const char* GLEInterface::getInitialPostScript() {
@@ -488,7 +494,7 @@ void GLEInterface::renderText(GLETextDO* text, GLEPropertyStore* prop) {
 	saved_state.save();
 	g_clear();
 	g_resetfont();
-	g_scale(72.0/CM_PER_INCH,72.0/CM_PER_INCH);
+	g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
 	g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
 	dev->startRecording();
 	// Select the right color
@@ -502,12 +508,10 @@ void GLEInterface::renderText(GLETextDO* text, GLEPropertyStore* prop) {
 	g_set_line_width(0.02);
 	// Select the font
 	GLEFont* font = prop->getFontProperty(GLEDOPropertyFont);
-	if (font == NULL) font = getFont("rm");
-	if (font != NULL) {
-		GLEFontStyle style = (GLEFontStyle)prop->getIntProperty(GLEDOPropertyFontStyle);
-		if (font->hasStyle(style)) font = font->getStyle(style);
-		g_set_font(font->getIndex());
+	if (font == NULL) {
+		font = getFont("rm");
 	}
+	g_set_font(font->getIndex());
 	char* str = (char*)text->getTextC();
 	// Calculate the text's size
 	double x1, x2, y1, y2;
@@ -594,6 +598,10 @@ void GLEInterface::setCmdLineOptionString(const char* name, const char* value, i
 	getCmdLine()->setOptionString(string(name), string(value), arg);
 }
 
+bool GLEInterface::hasCmdLineOptionString(const char* name) {
+	return getCmdLine()->hasOption(string(name));
+}
+
 string GLEInterface::getGhostScriptLocation() {
 	ConfigSection* tools = g_Config.getSection(GLE_CONFIG_TOOLS);
 #ifdef __WIN32__
@@ -690,10 +698,7 @@ void GLEInterface::initTextProperties(GLEPropertyStore* prop) {
 	g_get_hei(&fontsize);
 	prop->setRealProperty(GLEDOPropertyFontSize, fontsize);
 	g_get_font(&font);
-	// this lookup should be removed if "core.cpp" keeps pointer to GLEFont
-	font = getFontIndex(font);
-	if (font == -1) font = 0;
-	prop->setFontProperty(GLEDOPropertyFont, getFont(font));
+	prop->setFontProperty(GLEDOPropertyFont, getFontIndex(font));
 }
 
 extern GLEGlobalSource* g_Source;
@@ -731,6 +736,29 @@ int GLEInterface::copyFile(const string& from, const string& to, string* err) {
 	return GLECopyFile(from, to, err);
 }
 
+void GLEInterface::convertPDFToImage(char* pdfData,
+                                     int pdfLength,
+                                     double resolution,
+                                     int device,
+                                     int options,
+                                     gle_write_func writeFunc,
+                                     void* closure)
+{
+	gle_convert_pdf_to_image(pdfData,
+			                 pdfLength,
+			                 resolution,
+			                 device,
+			                 options,
+			                 writeFunc,
+			                 closure);
+}
+
+bool GLEReadFileOrGZIPTxt(const std::string& name, std::string* result);
+
+bool GLEInterface::readFileOrGZIPTxt(const char* name, std::string* result) {
+   return GLEReadFileOrGZIPTxt(std::string(name), result);
+}
+
 GLEFileLocation::GLEFileLocation() {
 	m_Flags = 0;
 }
@@ -814,12 +842,13 @@ void GLEFileLocation::copy(const GLEFileLocation* other) {
 }
 
 void GLEFileLocation::addExtension(const char* ext) {
-	m_Ext = ext;
+	const char* myExt = ext[0] == '.' ? (ext + 1) : ext;
+	m_Ext = myExt;
 	m_FullPath += ".";
-	m_FullPath += ext;
+	m_FullPath += myExt;
 	if ((m_Flags & (GLE_FILELOCATION_IS_STDIN | GLE_FILELOCATION_IS_STDOUT | GLE_FILELOCATION_IS_ILLEGAL)) == 0) {
 		m_Name += ".";
-		m_Name += ext;
+		m_Name += myExt;
 	}
 }
 
@@ -929,10 +958,10 @@ void GLEDrawObject::applyTransformationPt(GLEPoint* pt, bool dir) {
 	if (dir) {
 		double devx, devy;
 		g_dev(pt->getX(), pt->getY(), &devx, &devy);
-		pt->setXY((devx-1.0)/72.0*CM_PER_INCH, (devy-1.0)/72.0*CM_PER_INCH);
+		pt->setXY((devx-1.0)/PS_POINTS_PER_INCH*CM_PER_INCH, (devy-1.0)/PS_POINTS_PER_INCH*CM_PER_INCH);
 	} else {
 		double xp, yp;
-		g_undev(pt->getX()*72.0/CM_PER_INCH+1.0, pt->getY()*72.0/CM_PER_INCH+1.0, &xp, &yp);
+		g_undev(pt->getX()*PS_POINTS_PER_INCH/CM_PER_INCH+1.0, pt->getY()*PS_POINTS_PER_INCH/CM_PER_INCH+1.0, &xp, &yp);
 		pt->setXY(xp, yp);
 	}
 }
@@ -991,12 +1020,6 @@ GLEObjectDOConstructor::GLEObjectDOConstructor(GLESub* sub) {
 	linecap->addValue("square", GLELineCapSquare);
 	add(linecap);
 	add(new GLEPropertyFont("Font"));
-	GLEPropertyNominal* fontstyle = new GLEPropertyNominal("Font style", GLEPropertyTypeInt, GLEDOPropertyFontStyle);
-	fontstyle->addValue("roman", GLEFontStyleRoman);
-	fontstyle->addValue("bold", GLEFontStyleBold);
-	fontstyle->addValue("italic", GLEFontStyleItalic);
-	fontstyle->addValue("bold+italic", GLEFontStyleBoldItalic);
-	add(fontstyle);
 	add(new GLEPropertyHei("Font size"));
 }
 
@@ -1062,7 +1085,7 @@ void GLEScript::cleanUp() {
 	m_Parser = NULL;
 	m_Polish = NULL;
 	m_Pcode = NULL;
-	m_PostScript.resize(0);
+	m_PostScriptCode.resize(0);
 }
 
 GLEDrawObject* GLEScript::newGLEObject(GLEDrawObjectType type) {
@@ -1146,24 +1169,33 @@ void GLEScript::updateObjectDOConstructors() {
 
 const char* GLEScript::getPostScriptCode()
 {
-	return m_PostScript.c_str();
+	return m_PostScriptCode.c_str();
 }
 
-string* GLEScript::getRecordedBytesBuffer(int /* device */)
+string* GLEScript::getRecordedBytesBuffer(int device)
+{
+	if (device == GLE_DEVICE_EPS) {
+		return &m_PostScriptCode;
+	} else if (device == GLE_DEVICE_PDF) {
+		return &m_PDFCode;
+	} else {
+		CUtilsAssert(false);
+		return 0;
+	}
+}
+
+GLEHasArrowBase::GLEHasArrowBase():
+	m_Arrow(GLEHasArrowNone)
 {
-	return &m_PostScript;
 }
 
 GLELineDO::GLELineDO() {
-	m_Arrow = GLEHasArrowNone;
 }
 
 GLELineDO::GLELineDO(const GLEPoint& p1, const GLEPoint& p2) : m_P1(p1), m_P2(p2) {
-	m_Arrow = GLEHasArrowNone;
 }
 
 GLELineDO::GLELineDO(double x1, double y1, double x2, double y2) : m_P1(x1,y1), m_P2(x2,y2) {
-	m_Arrow = GLEHasArrowNone;
 }
 
 GLELineDO::~GLELineDO() {
@@ -1184,27 +1216,24 @@ bool GLELineDO::needsAMove(GLEPoint& pt) {
 	return true;
 }
 
+namespace {
+
+	void addArrowToCode(ostream& str, GLEHasArrow arrow) {
+		if (arrow == (GLEHasArrowStart | GLEHasArrowEnd)) {
+			str << " arrow both";
+		} else if (arrow == GLEHasArrowStart) {
+			str << " arrow start";
+		} else if (arrow == GLEHasArrowEnd) {
+			str << " arrow end";
+		}
+	}
+
+}
+
 void GLELineDO::createGLECode(string& code) {
 	ostringstream str;
-// FIXME: if we need this implementation -> need to detect amove at start of command
-//        if such amove is there -> need to include it again!!
-//        only don't include amove if position is correct from _previous_ line
-/*	if (getP1().approx(ox, oy)) {
-		str << "aline " << getP2().getX() << " " << getP2().getY();
-	} else if (getP2().approx(ox, oy)) {
-		str << "aline " << getP1().getX() << " " << getP1().getY();
-		m_P1.swap(m_P2);
-	} else {*/
-		str << "aline " << getP2().getX() << " " << getP2().getY();
-/*	}*/
-	GLEHasArrow arrow = getArrow();
-	if (arrow == (GLEHasArrowStart | GLEHasArrowEnd)) {
-		str << " arrow both";
-	} else if (arrow == GLEHasArrowStart) {
-		str << " arrow start";
-	} else if (arrow == GLEHasArrowEnd) {
-		str << " arrow end";
-	}
+	str << "aline " << getP2().getX() << " " << getP2().getY();
+	addArrowToCode(str, getArrow());
 	code = str.str();
 }
 
@@ -1361,13 +1390,14 @@ bool GLEArcDO::needsAMove(GLEPoint& pt) {
 
 void GLEArcDO::createGLECode(string& code) {
 	ostringstream str;
-	double angle2 = getNormalizedAngle2();
+	double angle2 = g_arc_normalized_angle2(m_Angle1, m_Angle2);
 	if (isCircle()) {
 		str << "arc " << m_Rx << " " << m_Angle1 << " " << angle2;
 	} else {
 
 		str << "elliptical_arc " << m_Rx << " " << m_Ry << " " << m_Angle1 << " " << angle2;
 	}
+	addArrowToCode(str, getArrow());
 	code = str.str();
 }
 
@@ -1376,30 +1406,23 @@ void GLEArcDO::updateBoundingBox() {
 }
 
 GLEDrawObject* GLEArcDO::clone() {
-	return new GLEArcDO(m_Center.getX(), m_Center.getY(), m_Rx, m_Ry, m_Angle1, m_Angle2);
+	GLEArcDO* result = new GLEArcDO(m_Center.getX(), m_Center.getY(), m_Rx, m_Ry, m_Angle1, m_Angle2);
+	result->setArrow(getArrow());
+	return result;
 }
 
 bool GLEArcDO::approx(GLEDrawObject* other) {
 	GLEArcDO* otherc = (GLEArcDO*)other;
 	return GLEEllipseDO::approx(other) &&
 	       fabs(m_Angle1 - otherc->getAngle1()) < 1e-6 &&
-	       fabs(m_Angle2 - otherc->getAngle2()) < 1e-6;
+	       fabs(m_Angle2 - otherc->getAngle2()) < 1e-6 &&
+	       getArrow() == otherc->getArrow();
 }
 
 void GLEArcDO::normalize() {
-	m_Angle2 = getNormalizedAngle2();
+	m_Angle2 = g_arc_normalized_angle2(m_Angle1, m_Angle2);
 }
 
-double GLEArcDO::getNormalizedAngle2() {
-	if (m_Angle2 < m_Angle1) {
-		// Correctly normalize a2
-		return m_Angle2 + 360 * ceil((m_Angle1 - m_Angle2)/360.0);
-	} else {
-		return m_Angle2;
-	}
-}
-
-
 GLEPoint& GLEArcDO::getPoint1(GLEPoint& pt) {
 	pt.set(m_Center);
 	pt.add(m_Rx*cos(m_Angle1*GLE_PI/180.0), m_Ry*sin(m_Angle1*GLE_PI/180.0));
@@ -1414,7 +1437,7 @@ GLEPoint& GLEArcDO::getPoint2(GLEPoint& pt) {
 
 GLEPoint& GLEArcDO::getPointMid(GLEPoint& pt) {
 	pt.set(m_Center);
-	double angle2 = getNormalizedAngle2();
+	double angle2 = g_arc_normalized_angle2(m_Angle1, m_Angle2);
 	double angleMid = (m_Angle1 + angle2)/2;
 	pt.add(m_Rx*cos(angleMid*GLE_PI/180.0), m_Ry*sin(angleMid*GLE_PI/180.0));
 	return pt;
@@ -1542,7 +1565,7 @@ void GLEObjectDO::getPSBoundingBox(GLERectangle* box) {
 	box->copy(m_ObjRep->getRectangle());
 	// Convert from points to cm
 	box->translate(-1.0, -1.0);
-	box->scale(CM_PER_INCH/72.0);
+	box->scale(CM_PER_INCH/PS_POINTS_PER_INCH);
 }
 
 bool GLEObjectDO::needsAMove(GLEPoint& pt) {
@@ -1623,7 +1646,7 @@ void GLEObjectDO::render() {
 		saved_state.save();
 		g_clear();
 		g_resetfont();
-		g_scale(72.0/CM_PER_INCH,72.0/CM_PER_INCH);
+		g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
 		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
 		dev->startRecording();
 		GLEPropertyStore* prop = getProperties();
@@ -1641,12 +1664,10 @@ void GLEObjectDO::render() {
 		g_set_line_width(prop->getRealProperty(GLEDOPropertyLineWidth));
 		// Select the font
 		GLEFont* font = prop->getFontProperty(GLEDOPropertyFont);
-		if (font == NULL) font = iface->getFont("rm");
-		if (font != NULL) {
-			GLEFontStyle style = (GLEFontStyle)prop->getIntProperty(GLEDOPropertyFontStyle);
-			if (font->hasStyle(style)) font = font->getStyle(style);
-			g_set_font(font->getIndex());
+		if (font == NULL) {
+			font = iface->getFont("rm");
 		}
+		g_set_font(font->getIndex());
 		newobj->enableChildObjects();
 		GLERun* run = script->getRun();
 		run->setDeviceIsOpen(true);
@@ -1751,11 +1772,24 @@ void GLEComposedObject::removeDeletedObjects() {
 GLEFont::GLEFont() {
 	m_Bold = m_Italic = m_BoldItalic = NULL;
 	m_Index = m_Number = 0;
+	m_Parent = 0;
 }
 
 GLEFont::~GLEFont() {
 }
 
+GLEFontStyle GLEFont::checkStyle(GLEFont* child) {
+	if (m_Bold.get() == child) {
+		return GLEFontStyleBold;
+	} else if (m_Italic.get() == child) {
+		return GLEFontStyleItalic;
+	} else if (m_BoldItalic.get() == child) {
+		return GLEFontStyleBoldItalic;
+	} else {
+		return GLEFontStyleRoman;
+	}
+}
+
 GLEFont* GLEFont::getStyle(GLEFontStyle style) {
 	switch (style) {
 		case GLEFontStyleRoman:
@@ -1787,34 +1821,89 @@ void GLEFont::setStyle(GLEFontStyle style, GLEFont* font) {
 	}
 }
 
-GLEColor::GLEColor() {
-	m_Red = m_Blue = m_Green = 0.0;
-	m_Transparent = true;
-	m_Name = NULL;
+GLEFillBase::GLEFillBase() {
 }
 
-GLEColor::GLEColor(double r, double g, double b) {
-	m_Red = r;
-	m_Green = g;
-	m_Blue = b;
-	m_Transparent = false;
-	m_Name = NULL;
+GLEFillBase::~GLEFillBase() {
+}
+
+GLEPatternFill::GLEPatternFill(int fillDescr) :
+	m_fillDescription(fillDescr),
+	m_background(new GLEColor())
+{
+	m_background->setHexValueGLE(GLE_COLOR_WHITE);
+}
+
+GLEPatternFill::~GLEPatternFill() {
+}
+
+GLEFillType GLEPatternFill::getFillType() {
+	return GLE_FILL_TYPE_PATTERN;
+}
+
+GLEFillBase* GLEPatternFill::clone() {
+	GLEPatternFill* result = new GLEPatternFill(m_fillDescription);
+	result->setBackground(m_background->clone());
+	return result;
+}
+
+GLEColor::GLEColor() :
+	m_Name(0)
+{
+	setGray(0);
+}
+
+GLEColor::GLEColor(double r, double g, double b) :
+	m_Name(0)
+{
+	setRGB(r, g, b);
+}
+
+GLEColor::GLEColor(double r, double g, double b, double a) :
+	m_Name(0)
+{
+	setRGBA(r, g, b, a);
+}
+
+GLEColor::GLEColor(double gray) :
+	m_Name(0)
+{
+	setGray(gray);
 }
 
 GLEColor::~GLEColor() {
 	if (m_Name != NULL) delete m_Name;
 }
 
+GLEColor* GLEColor::clone() {
+	GLEColor* result = new GLEColor(m_Red, m_Green, m_Blue, m_Alpha);
+	result->setTransparent(isTransparent());
+	result->setName(m_Name);
+	if (isFill()) {
+		result->setFill(getFill()->clone());
+	}
+	return result;
+}
+
 int GLEColor::getType() {
 	return GLEObjectTypeColor;
 }
 
+bool GLEColor::equalsApprox(GLEColor* other) {
+	return equals_rel_fine(m_Red, other->m_Red) &&
+		   equals_rel_fine(m_Green, other->m_Green) &&
+		   equals_rel_fine(m_Blue, other->m_Blue) &&
+		   equals_rel_fine(m_Alpha, other->m_Alpha) &&
+		   m_Transparent == other->m_Transparent;
+}
+
 bool GLEColor::equals(GLEDataObject* obj) {
 	if (obj->getType() != GLEObjectTypeColor) return false;
 	GLEColor* other = (GLEColor*)obj;
 	return m_Red == other->m_Red &&
 	       m_Green == other->m_Green &&
 	       m_Blue == other->m_Blue &&
+	       m_Alpha == other->m_Alpha &&
 	       m_Transparent == other->m_Transparent;
 }
 
@@ -1838,7 +1927,11 @@ void GLEColor::toString(ostream& out) {
 			}
 		}
 		if (!found) {
-			out << "rgb255(" << (int)getRedI() << "," << (int)getGreenI() << "," << (int)getBlueI() << ")";
+			if (hasAlpha()) {
+				out << "rgba255(" << (int)getRedI() << "," << (int)getGreenI() << "," << (int)getBlueI() << "," << (int)getAlphaI() << ")";
+			} else {
+				out << "rgb255(" << (int)getRedI() << "," << (int)getGreenI() << "," << (int)getBlueI() << ")";
+			}
 		}
 	}
 }
@@ -1848,37 +1941,92 @@ const char* GLEColor::getName() {
 }
 
 void GLEColor::setName(const string& name) {
-	if (m_Name != NULL) delete m_Name;
+	delete m_Name;
 	m_Name = new string(name);
 }
 
+void GLEColor::setName(const string* name) {
+	delete m_Name;
+	if (name != 0) {
+		m_Name = new std::string(*name);
+	} else {
+		m_Name = 0;
+	}
+}
+
 void GLEColor::setRGB(double r, double g, double b) {
 	m_Red = r; m_Green = g; m_Blue = b;
+	m_Alpha = 1.0;
+	m_Transparent = false;
+}
+
+void GLEColor::setRGBA(double r, double g, double b, double a) {
+	m_Red = r; m_Green = g; m_Blue = b; m_Alpha = a;
 	m_Transparent = false;
 }
 
 void GLEColor::setRGB255(int r, int g, int b) {
-	m_Red   = ((double)r) / 255.0;
-	m_Green = ((double)g) / 255.0;
-	m_Blue  = ((double)b) / 255.0;
-	m_Transparent = false;
+	setRGB(((double)r) / 255.0, ((double)g) / 255.0, ((double)b) / 255.0);
+}
+
+void GLEColor::setRGB255(double r, double g, double b) {
+	setRGB(r / 255.0, g / 255.0, b / 255.0);
+}
+
+void GLEColor::setRGBA255(double r, double g, double b, double a) {
+	setRGBA(r / 255.0, g / 255.0, b / 255.0, a / 255.0);
+}
+
+void GLEColor::setGray(double gray) {
+	setRGB(gray, gray, gray);
 }
 
 void GLEColor::setHexValue(unsigned int v) {
 	unsigned int red   = (v >> 16) & 0xFF;
 	unsigned int green = (v >> 8) & 0xFF;
 	unsigned int blue  = v & 0xFF;
-	m_Red   = ((double)red) / 255.0;
-	m_Green = ((double)green) / 255.0;
-	m_Blue  = ((double)blue) / 255.0;
-	// cout << "RGB(" << red << "," << green << "," << blue << ") -> ";
-	// cout << "RGB(" << m_Red << "," << m_Green << "," << m_Blue << ")" << endl;
-	m_Transparent = false;
+	setRGB(((double)red) / 255.0, ((double)green) / 255.0, ((double)blue) / 255.0);
+}
+
+void GLEColor::setDoubleEncoding(double v) {
+	union {
+		double d;
+		int l[2];
+	} both;
+	both.d = v;
+	setHexValueGLE(both.l[0]);
+	m_Alpha = double(both.l[1]) / 255.0;
+}
+
+double GLEColor::getDoubleEncoding() {
+	union {
+		double d;
+		int l[2];
+	} both;
+	both.l[0] = getHexValueGLE();
+	both.l[1] = getAlphaI();
+	return both.d;
+}
+
+void GLEColor::setHexValueGLE(unsigned int hexValue) {
+	if (hexValue == GLE_FILL_CLEAR) {
+		setGray(0);
+		m_Fill = 0;
+		setTransparent(true);
+	} else if ((hexValue & 0X02000000) != 0) {
+		setGray(0);
+		setFill(new GLEPatternFill(hexValue));
+	} else {
+		setHexValue(hexValue);
+	}
 }
 
 unsigned int GLEColor::getHexValueGLE() {
 	if (isTransparent()) {
 		return GLE_FILL_CLEAR;
+	} else if (isFill() && getFill()->getFillType() == GLE_FILL_TYPE_PATTERN) {
+		GLEPatternFill* myFill = static_cast<GLEPatternFill*>(getFill());
+		return myFill->getFillDescription();
 	} else {
 		unsigned int red = float_to_color_comp(m_Red);
 		unsigned int green = float_to_color_comp(m_Green);
@@ -1887,6 +2035,10 @@ unsigned int GLEColor::getHexValueGLE() {
 	}
 }
 
+double GLEColor::getGray() {
+	return (getRed() * 3.0 + getGreen() * 2.0 + getBlue()) / 6.0;
+}
+
 GLEPropertyStore::GLEPropertyStore(GLEPropertyStoreModel* model) {
 	m_Model = model;
 	m_Values.ensure(model->getNumberOfProperties());
@@ -2069,14 +2221,9 @@ GLEPropertyColor::~GLEPropertyColor() {
 }
 
 bool GLEPropertyColor::isEqualToState(GLEPropertyStore* store) {
-	rgb01 rgb;
-	colortyp color;
-	g_get_colortyp(&color);
-	g_colortyp_to_rgb01(&color, &rgb);
+	GLERC<GLEColor> color(g_get_color());
 	GLEColor* gle_color = store->getColorProperty(this);
-	return equals_rel_fine(rgb.red, gle_color->getRed()) &&
-	       equals_rel_fine(rgb.blue, gle_color->getBlue()) &&
-	       equals_rel_fine(rgb.green, gle_color->getGreen());
+	return color->equalsApprox(gle_color);
 }
 
 void GLEPropertyColor::updateState(GLEPropertyStore* store) {
@@ -2090,17 +2237,8 @@ GLEPropertyFillColor::~GLEPropertyFillColor() {
 }
 
 bool GLEPropertyFillColor::isEqualToState(GLEPropertyStore* store) {
-	rgb01 rgb;
-	colortyp color;
-	g_get_fill_colortyp(&color);
-	GLEColor* gle_color = store->getColorProperty(this);
-	if (gle_color->isTransparent() != (color.l == (int)GLE_FILL_CLEAR)) {
-		return false;
-	}
-	g_colortyp_to_rgb01(&color, &rgb);
-	return equals_rel_fine(rgb.red, gle_color->getRed()) &&
-	       equals_rel_fine(rgb.blue, gle_color->getBlue()) &&
-	       equals_rel_fine(rgb.green, gle_color->getGreen());
+	GLERC<GLEColor> curr_fill(g_get_fill());
+	return curr_fill->equalsApprox(store->getColorProperty(this));
 }
 
 void GLEPropertyFillColor::updateState(GLEPropertyStore* store) {
@@ -2221,12 +2359,7 @@ void GLEPropertyArrowAngle::updateState(GLEPropertyStore* store) {
 }
 
 void GLEInitColorProperty(GLEPropertyStore* prop) {
-	rgb01 rgb;
-	colortyp color;
-	g_get_colortyp(&color);
-	g_colortyp_to_rgb01(&color, &rgb);
-	GLEColor* gcolor = new GLEColor(rgb.red, rgb.green, rgb.blue);
-	prop->setColorProperty(GLEDOPropertyColor, gcolor);
+	prop->setColorProperty(GLEDOPropertyColor, g_get_color()->clone());
 }
 
 void GLEInitSimpleLineProperties(GLEPropertyStore* prop) {
@@ -2292,15 +2425,6 @@ void GLEInitArrowProperties(GLEPropertyStore* prop) {
 }
 
 void GLEInitShapeFillColor(GLEPropertyStore* prop) {
-	rgb01 rgb;
-	colortyp color;
-	g_get_fill((int*)&color);
-	GLEColor* gcolor = new GLEColor();
-	if (color.l == (int)GLE_FILL_CLEAR) {
-		gcolor->setTransparent(true);
-	} else {
-		g_colortyp_to_rgb01(&color, &rgb);
-		gcolor->setRGB(rgb.red, rgb.green, rgb.blue);
-	}
-	prop->setColorProperty(GLEDOPropertyFillColor, gcolor);
+	GLERC<GLEColor> gcolor(g_get_fill());
+	prop->setColorProperty(GLEDOPropertyFillColor, gcolor->clone());
 }
