@@ -95,19 +95,19 @@ protected:
 	double m_Add;
 	bool m_IsRound;
 	double m_Round;
-	int m_Fill;
+	GLERC<GLEColor> m_Fill;
 public:
 	GLEBox();
 	GLEBox(const GLEBox& box);
-	void setFill(int fill);
+	void setFill(const GLERC<GLEColor>& fill);
 	void setRound(double round);
 	void draw(GLERun* run, double x1, double y1, double x2, double y2);
 	inline void setStroke(bool stroke) { m_HasStroke = stroke; }
 	inline bool hasStroke() const { return m_HasStroke; }
 	inline void setReverse(bool reverse) { m_HasReverse = reverse; }
 	inline bool hasReverse() const { return m_HasReverse; }
-	inline bool isFilled() const { return m_Fill != (int)GLE_FILL_CLEAR; }
-	inline int getFill() const { return m_Fill; }
+	inline bool isFilled() const { return !m_Fill->isTransparent(); }
+	inline GLERC<GLEColor> getFill() const { return m_Fill; }
 	inline bool isRound() const { return m_IsRound; }
 	inline double getRound() const { return m_Round; }
 	inline void setNamePtr(const char* name) { m_Name = name; }
@@ -160,12 +160,12 @@ public:
 GLEBoxStack GLEBoxStack::m_Instance;
 
 GLEStoredBox* box_start(void);
-bool box_end(void) throw(ParserError);
 
 extern int this_line;
 extern int trace_on;
 static int path_clip[4],path_stroke[4];
-static int path_fill[4],path_fill_backup[4];
+static GLERC<GLEColor> path_fill[4];
+static GLERC<GLEColor> path_fill_backup[4];
 static double path_x[4],path_y[4];
 static int npath;
 #define true (!false)
@@ -470,6 +470,7 @@ GLERun::GLERun(GLEScript* script, GLEFileLocation* outfile) {
 	m_OutFile = outfile;
 	m_Vars = getVarsInstance();
 	m_CrObj = new GLEObjectRepresention();
+	m_blockTypes = 0;
 	for (int i = 0; i < GLE_KW_NB; i++) {
 		m_AllowBeforeSize[i] = false;
 	}
@@ -517,6 +518,14 @@ void GLERun::setDeviceIsOpen(bool open) {
 	done_open = open;
 }
 
+void GLERun::setBlockTypes(GLEBlocks* blocks) {
+	m_blockTypes = blocks;
+}
+
+GLEBlocks* GLERun::getBlockTypes() {
+	return m_blockTypes;
+}
+
 void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, int *pend, bool& mkdrobjs) throw(ParserError) {
 /* srclin = The source line number */
 /* pcode =  a pointer to the pcode output buffer */
@@ -530,6 +539,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 	static char ss[255],ss2[80],ss1[90];
 	static bool jump_back = false;
 	GLEPoint orig;
+	GLERC<GLEColor> colorBackup;
 	*pend = 0;
 	this_line = *srclin;
 	while (cp < plen) {
@@ -626,6 +636,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			}
 			if (mkdrobjs) {
 				GLEArcDO drawobj(ox, oy, r, a1, a2);
+				drawobj.setArrow((GLEHasArrow)marrow);
 				handleNewDrawObject(&drawobj, mkdrobjs);
 			} else {
 				g_arc(r, a1, a2, ox, oy, marrow);
@@ -653,15 +664,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_get_xy(&path_x[npath],&path_y[npath]);
 					path_stroke[npath] = *(pcode + cp);
 					ptr = *(pcode + ++cp);
-					path_fill[npath] = GLE_FILL_CLEAR;
-					g_get_fill(&path_fill_backup[npath]);
+					path_fill_backup[npath] = g_get_fill();
 					path_clip[npath] = *(pcode + cp + 1);
 					if (!path_clip[npath]) {
-						g_get_fill(&path_fill[npath]);
+						path_fill[npath] = g_get_fill();
+					} else {
+						path_fill[npath] = g_get_fill_clear();
 					}
 					if (ptr) {
 						readvalp(z,pcode+cp+ptr);
-						memcpy(&path_fill[npath],&z,4);
+						path_fill[npath] = color_from_double_encoding(z);
 					}
 					cp++;
 					g_set_path(true);
@@ -669,6 +681,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					break;
 				case 2: /* BOX add, fill, nobox, name, round */
 					{
+						g_drobj.push_back(mkdrobjs);
 						GLEStoredBox* box = box_start();
 						ptr = *(pcode + cp);
 						if (ptr) {
@@ -678,8 +691,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						ptr = *(pcode + ++cp);
 						if (ptr) {
 							readvalp(z,pcode+cp+ptr);
-							memcpy(&both.d,&z,sizeof(z));
-							box->setFill(both.l);
+							box->setFill(color_from_double_encoding(z));
 						}
 						if (*(pcode + ++cp)) {
 							box->setStroke(false);
@@ -697,6 +709,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						if (box->isFilled() && !g_is_dummy_device()) {
 							/* Draw over filled box, measure during first pass */
 							box->setDevice(g_set_dummy_device());
+							mkdrobjs = false;
 						}
 					}
 					break;
@@ -752,17 +765,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				case 18: /* tab  (tabbing, table) */
 					begin_tab(srclin,pcode,&cp);
 					break;
-				case 10: /* graph */
-					begin_graph(srclin,pcode,&cp);
+				case GLE_OPBEGIN_KEY:
+				case GLE_OPBEGIN_GRAPH:
+				case GLE_OPBEGIN_SURF:
+					getBlockTypes()->getBlock(i)->beginExecuteBlock(sline, pcode, &cp);
 					break;
 				case 11: /* xaxis */
 				case 12: /* yaxis */
 				case 13: /* x2axis */
 				case 14: /* y2axis */
 					break;
-				case 16: /* KEY */
-					begin_key(srclin,pcode,&cp);
-					break;
 				case 19: /* begin  clip */
 					g_beginclip();
 					break;
@@ -779,9 +791,6 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				case 23: /* tex preamble */
 					begin_tex_preamble(srclin,pcode,&cp);
 					break;
-				case 24: /* surface */
-					begin_surface(srclin,pcode,&cp);
-					break;
 				case 25: /* letz */
 					begin_letz(srclin,pcode,&cp);
 					break;
@@ -815,6 +824,10 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						*srclin = j;	/* skip past the subroutine */
 					}
 					break;
+				case OP_BEGIN_LENGTH:
+					readlong(jj);
+					begin_length(jj);
+					break;
 				default: /* error  */
 					g_throw_parser_error("illegal begin option code: ", i);
 					break;
@@ -835,14 +848,12 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				mask_just = *(pcode + cp);
 				g_dojust(&ox,&oy,&x,&y,mask_just);
 				if (g_is_filled()) {
-					g_get_fill(&jj);
-					box.setFill(jj);
+					box.setFill(g_get_fill());
 				}
 				ptr = *(pcode + ++cp);
 				if (ptr) {
 					readvalp(z,pcode+cp+ptr);
-					memcpy(&both.d,&z,sizeof(z));
-					box.setFill(both.l);
+					box.setFill(color_from_double_encoding(z));
 				}
 				if (*(pcode + ++cp)) {
 					box.setStroke(false);
@@ -877,12 +888,11 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			g_dojust(&ox,&oy,&x,&y,mask_just);
 			g_move(ox,oy);
 			mask_nostroke = *(pcode + cp++);
-			g_get_fill(&jj);
+			colorBackup = g_get_fill();
 			ptr_fill = *(pcode + cp);
 			if (ptr_fill) {
 				readvalp(z,pcode + cp + ptr_fill);
-				memcpy(&both.l,&z,4);
-				g_set_fill(both.l);
+				g_set_fill(color_from_double_encoding(z));
 			}
 			if (mkdrobjs) {
 				GLEEllipseDO drawobj(ox, oy, r);
@@ -891,7 +901,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				if (g_is_filled()) g_circle_fill(r);
 				if (!mask_nostroke) g_circle_stroke(r);
 			}
-			g_set_fill(jj);
+			g_set_fill(colorBackup);
 			g_move(orig);
 			break;
 		case 70:  /* ELLIPSE */
@@ -905,13 +915,12 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			y = oy + ry;
 			g_dojust(&ox,&oy,&x,&y,mask_just);
 			g_move(ox,oy);
-			g_get_fill(&jj);
+			colorBackup = g_get_fill();
 			mask_nostroke = *(pcode + cp++);
 			ptr_fill = *(pcode + cp);
 			if (ptr_fill) {
 				readvalp(z,pcode + cp + ptr_fill);
-				memcpy(&both.l,&z,4);
-				g_set_fill(both.l);
+				g_set_fill(color_from_double_encoding(z));
 			}
 			if (mkdrobjs) {
 				GLEEllipseDO drawobj(ox, oy, rx, ry);
@@ -920,7 +929,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				if (g_is_filled()) g_ellipse_fill(rx,ry);
 				if (!mask_nostroke) g_ellipse_stroke(rx,ry);
 			}
-			g_set_fill(jj);
+			g_set_fill(colorBackup);
 			g_move(orig);
 			break;
 		case 71: /* ELLIPTICAL_ARC  */
@@ -983,7 +992,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			readlong(jj);
 			switch (jj) {
 			  case 1: /* end path  (stroke,fill,clip) */
-				if (path_fill[npath] != (int)GLE_FILL_CLEAR) {
+				if (!path_fill[npath]->isTransparent()) {
 					g_set_fill(path_fill[npath]);
 					g_fill();
 				}
@@ -1001,6 +1010,10 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			  case 2: /* end box */
 				cp++;
 				readlong(jj);   /* jump to line */
+				if (!last_box()->isSecondPass() && g_drobj.size() != 0) {
+					mkdrobjs = g_drobj.back();
+					g_drobj.pop_back();
+				}
 				if (box_end()) {
 					*pend = 0;
 					*srclin = jj;
@@ -1037,6 +1050,14 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				cp++;
 				end_object();
 				break;
+			  case GLE_OPBEGIN_KEY:
+			  case GLE_OPBEGIN_GRAPH:
+			  case GLE_OPBEGIN_SURF:
+				getBlockTypes()->getBlock(jj)->endExecuteBlock();
+				break;
+			  case OP_BEGIN_LENGTH:
+				end_length();
+			    break;
 			  default :
 				get_global_parser()->get_block_type(jj, temp_str);
 				g_throw_parser_error("invalid end of block type '", temp_str.c_str(), "'");
@@ -1431,28 +1452,25 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_set_font(both.l);
 					break;
 				  case 3: /* justify */
-					readlong(jj);
-					g_set_just(jj);
+					readval(x);
+					memcpy(&both.l,&x,4);
+					g_set_just(both.l);
 					break;
 				  case 4: /* color */
 					readval(x);
-					memcpy(&both.l,&x,4);
-					g_set_color(both.l);
+					g_set_color(color_from_double_encoding(x));
 					break;
 				  case OP_SET_BACKGROUND: /* background */
 					readval(x);
-					memcpy(&both.l,&x,4);
-					g_set_background(both.l);
+					g_set_background(color_from_double_encoding(x));
 					break;
 				  case OP_SET_FILL: /* fill */
 					readval(x);
-					memcpy(&both.l,&x,4);
-					g_set_fill(both.l);
+					g_set_fill(color_from_double_encoding(x));
 					break;
 				  case OP_SET_FILL_PATTERN: /* fill pattern */
 					readval(x);
-					memcpy(&both.l,&x,4);
-					g_set_fill_pattern(both.l);
+					g_set_fill_pattern(color_from_double_encoding(x));
 					break;
 				  case 5: /* dashlen */
 					readval(x);
@@ -1548,7 +1566,6 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			} else {
 				g_text(temp_str);
 			}
-			g_get_bounds(&x1,&y1,&x2,&y2);
 			break;
 		  case 60: /* DEFMARKER */
 			break;
@@ -1656,6 +1673,10 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			readval(x);
 			GLESleep((int)floor(x*1000+0.5));
 			break;
+		  case GLE_KW_BLOCK_COMMAND:
+			readlong(jj); /* block type */
+			getBlockTypes()->getBlock(jj)->executeLine(sline);
+			break;
 		  default :
 		  	byte_code_error(PCODE_UNKNOWN_COMMAND);
 		}
@@ -1665,25 +1686,28 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 	}
 }
 
-GLEBox::GLEBox() {
-	m_Name = NULL;
-	m_HasStroke = true;
-	m_HasReverse = false;
-	m_Add = 0.0;
-	m_IsRound = false;
-	m_Fill = GLE_FILL_CLEAR;
+GLEBox::GLEBox():
+	m_Name(NULL),
+	m_HasStroke(true),
+	m_HasReverse(false),
+	m_Add(0.0),
+	m_IsRound(false),
+	m_Fill(g_get_fill_clear())
+{
 }
 
-GLEBox::GLEBox(const GLEBox& box) {
-	m_Name = box.getName();
-	m_HasStroke = box.hasStroke();
-	m_HasReverse = box.hasReverse();
-	m_Add = box.getAdd();
-	m_IsRound = box.isRound(); m_Round = box.getRound();
-	m_Fill = box.getFill();
+GLEBox::GLEBox(const GLEBox& box):
+	m_Name(box.getName()),
+	m_HasStroke(box.hasStroke()),
+	m_HasReverse(box.hasReverse()),
+	m_Add(box.getAdd()),
+	m_IsRound(box.isRound()),
+	m_Round(box.getRound()),
+	m_Fill(box.getFill())
+{
 }
 
-void GLEBox::setFill(int fill) {
+void GLEBox::setFill(const GLERC<GLEColor>& fill) {
 	m_Fill = fill;
 }
 
@@ -1703,13 +1727,12 @@ void do_arcto(double x1, double y1, double x2, double y2, double r) {
 }
 
 void GLEBox::draw(GLERun* run, double x1, double y1, double x2, double y2) {
-	int cur_fill;
 	double ox, oy;
 	GLERectangle rect(x1, y1, x2, y2);
 	rect.normalize();
 	rect.grow(getAdd());
 	g_get_xy(&ox, &oy);
-	g_get_fill(&cur_fill);
+	GLERC<GLEColor> cur_fill(g_get_fill());
 	if (isRound()) {
 		int oldjoin;
 		g_get_line_join(&oldjoin);
@@ -1775,6 +1798,14 @@ GLEStoredBox* box_start() {
 	return box;
 }
 
+GLEStoredBox* GLERun::last_box() throw (ParserError) {
+	GLEBoxStack* stack = GLEBoxStack::getInstance();
+	if (stack->size() <= 0) {
+		g_throw_parser_error("too many end boxes");
+	}
+	return stack->lastBox();
+}
+
 bool GLERun::box_end() throw (ParserError) {
 	double x1, y1, x2, y2;
 	GLEBoxStack* stack = GLEBoxStack::getInstance();
@@ -2285,7 +2316,7 @@ void GLERun::begin_object(const char* name, GLESub* sub) throw (ParserError) {
 	}
 	/* store graphics state */
 	g_move(0.0, 0.0);
-	gmodel* state = (gmodel*) myallocz(SIZEOFSTATE);
+	gmodel* state = new gmodel();
 	g_get_state(state);
 	dynsub->setState(state);
 	/* Do not actually draw */
@@ -2328,6 +2359,31 @@ void GLERun::end_object() throw (ParserError) {
 	stack->removeBox();
 }
 
+void GLERun::begin_length(int var)
+{
+	GLELengthBlock previous;
+	GLECore* core = g_get_core();
+	previous.varIndex = var;
+	previous.wasEnabled = core->isComputingLength();
+	previous.previousValue = core->getTotalLength();
+	m_lengthBlocks.push_back(previous);
+	core->setComputingLength(true);
+	core->setTotalLength(0.0);
+}
+
+void GLERun::end_length()
+{
+	GLECore* core = g_get_core();
+	CUtilsAssert(m_lengthBlocks.size() > 0);
+	CUtilsAssert(core->isComputingLength());
+	GLELengthBlock block(m_lengthBlocks.back());
+	m_lengthBlocks.pop_back();
+	double length = core->getTotalLength();
+	core->setComputingLength(block.wasEnabled);
+	core->setTotalLength(block.previousValue + length);
+	getVars()->setDouble(block.varIndex, length);
+}
+
 GLESubMap* GLERun::getSubroutines() {
 	return &g_Subroutines;
 }
