@@ -138,7 +138,7 @@ void text_gprint(int *in,int ilen);
 void fftext_block(const string& s,double width,int justify);
 void font_load(void);
 void tex_init(void);
-
+int g_font_fallback(int font);
 void decode_utf8_basic(string& sc);
 
 void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params);
@@ -219,9 +219,17 @@ IntStringHash m_Unicode;
 union both {float f;int l;} bth;
 #define outlong(v) *(out+((*lout)++)) = v
 #define outfloat(v) bth.f = v; *(out+((*lout)++)) = bth.l
-#define tofloat(fff) ((bth.l = fff),bth.f )
 #define tolong(fff) ((bth.f = fff),bth.l )
 
+float tofloat(int input) {
+	union both {
+		float f;
+		int l;
+	} bth;
+	bth.l = input;
+	return bth.f;
+}
+
 extern bool IS_INSTALL;
 extern CmdLineObj g_CmdLine;
 
@@ -233,6 +241,10 @@ void set_base_size() {
 	g_get_hei(&base_size);
 }
 
+void set_tex_font(int fnt) {
+	p_fnt = g_font_fallback(fnt);
+}
+
 void try_get_next_char(uchar **in, int *c1) {
 	*c1 = 0;
 	if (**in != 0) {
@@ -305,6 +317,7 @@ void text_topcode(uchar *in, int *out, int *lout) {
 		/* if next char is normal, then check for ligature and kern */
 norm_again:
 		w = 0;
+	    p_fnt = g_font_fallback(p_fnt);
 		cfont = get_core_font_ensure_loaded(p_fnt);
 		if (c2 != 0) {
 			if (!g_CmdLine.hasOption(GLE_OPT_NO_LIGATURES)) {
@@ -326,6 +339,7 @@ norm_again:
 		if (skip_space) break;
 		skip_space = true;
 		outlong(2);
+		p_fnt = g_font_fallback(p_fnt);
 		cfont = get_core_font_ensure_loaded(p_fnt);
 		outfloat(cfont->info.space*p_hei);
 		outfloat(cfont->info.space_stretch*p_hei*10*stretch_factor);
@@ -639,7 +653,7 @@ void tex_get_char_code(uchar** in, int* code) {
 
 void tex_draw_accent(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	double lef,dep,wid,hei;
-	double wid2,hei2,lef2,dep2,ww,h=0,cwid,cwid2;
+	double wid2,hei2,lef2,dep2,h=0,cwid,cwid2;
 	int ix,ix2;
 	int savefnt = p_fnt;
 	int newfnt = pass_font(params->getCStr1());
@@ -675,7 +689,6 @@ void tex_draw_accent(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	lef *= p_hei;  dep *= p_hei;
 	lef2 *= p_hei; dep2 *= p_hei;
 	if (hei2>p_hei*(3.6/8.0)) h = hei2-p_hei*(3.6/8.0);
-	ww = lef2+wid2;
 	if (m == NULL) {
 		p_fntchar(p_fnt,ix2);
 	} else {
@@ -685,7 +698,7 @@ void tex_draw_accent(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	p_move(-cwid2+lef2+wid2/2-wid/2+accent_x,h+accent_y);  /* cwid2/2 - cwid/2 */
 	p_fntchar(newfnt,ix);        /* cwid2/2 + cwid/2 */
 	p_move(-cwid+cwid2-lef2-wid2/2+wid/2-accent_x,-h-accent_y);
-	p_fnt = savefnt;
+	set_tex_font(savefnt);
 	font_load_metric(p_fnt);
 }
 
@@ -720,9 +733,17 @@ void tex_draw_accent_cmb(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	}
 }
 
+int select_font_encoding(int font, int encoding, const char* defaultFont) {
+	GLECoreFont* crfont = get_core_font_ensure_loaded(font);
+	if (crfont->info.encoding == encoding) {
+		return font;
+	} else {
+		return pass_font(defaultFont);
+	}
+}
+
 void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 	int ci;
-	int next_ci;
 	int ix;
 	char cmdstr[20];
 	double lef,wid,hei,dep,savehei;
@@ -762,7 +783,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		p_pcode(pbuff,plen);
 		/*printf("this P=[%s]\n",pm.s[0]);*/
 		p_move(0,-0.8*p_hei);
-		next_ci = find_primcmd(cmdstr);
+		find_primcmd(cmdstr);
 
 		/* do a look ahead to see if there is a superscript */
 		/* but keep in pointing to current spot */
@@ -828,6 +849,11 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		fontfam[i][k] = pass_font(params->getCStr2());
 		fontfamsz[i][k] = emtof(params->str3);
 		break;
+	  case tp_fontenc:
+		params->cmdParam2(in);
+		set_tex_font(select_font_encoding(p_fnt, atoi(params->getCStr1()), params->getCStr2()));
+		font_load_metric(p_fnt);
+		break;
 	  case tp_acccmb:
   		params->cmdParam4_swap34(in);
 		tex_draw_accent_cmb(in, params, out, lout);
@@ -894,7 +920,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		break;
 	  case tp_setfont:
 		params->cmdParam1(in);
-		p_fnt = pass_font(params->getCStr1());
+		set_tex_font(pass_font(params->getCStr1()));
 		font_load_metric(p_fnt);
 		break;
 	  case tp_presave:
@@ -1020,7 +1046,7 @@ void texint(const string& s, int *i) {
 
 void text_wrapcode(int *in,int ilen,double width) {
 	double cx=0,cy=0,p_hei,ax=0,y,cdep=0,chei=0;
-	int i,c,p_fnt,si,skline,saveii;
+	int i,c,crFont,si,skline,saveii;
 	double totstretch=0,totshrink=0,ls=0,gap=0,last_y,last_x,lastdep,last_stret,last_shrink;
 	int *pcr=0,last_space=0;
 	double setlen;
@@ -1042,8 +1068,8 @@ void text_wrapcode(int *in,int ilen,double width) {
 	  switch (*(in+i)) {
 	    case 1: /* char     font+char,wx    */
 		eat_glue = false;
-		p_fnt = (*(in+ ++i)) / 1024;
-		cfont = get_core_font_ensure_loaded(p_fnt);
+		crFont = g_font_fallback((*(in+ ++i)) / 1024);
+		cfont = get_core_font_ensure_loaded(crFont);
 		c = *(in+i) & 0x3ff;
 		if (cdep > cy + p_hei * cfont->cdata[c]->y1)
 			cdep = cy + p_hei * cfont->cdata[c]->y1;
@@ -1174,8 +1200,8 @@ void text_wrapcode(int *in,int ilen,double width) {
 		g_set_hei(p_hei);
 		break;
 	    case 9: /* font     p_fnt   */
-		p_fnt = *(in+ ++i);
-		font_load_metric(p_fnt);
+	    crFont = g_font_fallback(*(in+ ++i));
+		font_load_metric(crFont);
 
 		break;
 	    case 20: /*  nop  */
@@ -1275,7 +1301,7 @@ void set_glue(int *in,int ilen,double actual,double width,double stretch,double
 
 void text_draw(int *in,int ilen) {
 	double cx,cy,p_hei,x,y;
-	int i,c,p_fnt;
+	int i,c,crFont;
 	GLECoreFont* cfont;
 
 	dbg gprint("---TEXT DRAW, ilen = %d \n",ilen);
@@ -1289,14 +1315,14 @@ void text_draw(int *in,int ilen) {
 	for (i=0;i<ilen;i++) {
 	  switch (*(in+i)) {
 	    case 1: /* char     font+char,wx    */
-		p_fnt = (*(in+ ++i)) / 1024;
-		cfont = get_core_font_ensure_loaded(p_fnt);
+	    crFont = g_font_fallback((*(in+ ++i)) / 1024);
+		cfont = get_core_font_ensure_loaded(crFont);
 		c = *(in+i) & 0x3ff;
 		g_update_bounds(cx+p_hei*(cfont->cdata[c]->x1), cy+p_hei*(cfont->cdata[c]->y1));
 		g_update_bounds(cx+p_hei*(cfont->cdata[c]->x2), cy+p_hei*(cfont->cdata[c]->y2));
 		dp {
 			g_move(cx,cy);
-			g_char(p_fnt,c);
+			g_char(crFont,c);
 		}
 		cx += tofloat(*(in+ ++i));
 		break;
@@ -1330,8 +1356,8 @@ void text_draw(int *in,int ilen) {
 		g_set_hei(p_hei);
 		break;
 	    case 9: /* font     p_fnt   */
-		p_fnt = *(in+ ++i);
-		font_load_metric(p_fnt);
+	    crFont = g_font_fallback(*(in+ ++i));
+		font_load_metric(crFont);
 		break;
 	    case 10: /* Newline  x,y    (turned into a move) */
 		i += 2;
@@ -1371,18 +1397,16 @@ double tex_yend(void) {
 }
 
 void text_gprint(int *in,int ilen) {
-	int i,c,p_fnt,z;
-
+	int i,c,crFont;
 	for (i=0;i<ilen;i++) printf("%x ",*(in+i));
 	printf("\n");
-	z=0;
 	printf("# ");
 	double x = 0.0;
 	for (i=0;i<ilen;i++) {
 	  switch (*(in+i)) {
 	    case 1: /* char     font+char,wx    */
-		p_fnt = (*(in+ ++i)) / 1024;
-		font_load_metric(p_fnt);
+	    crFont = g_font_fallback((*(in+ ++i)) / 1024);
+		font_load_metric(crFont);
 		c = *(in+i) & 0x3ff;
 		x = tofloat(*(in+ ++i));
 		printf("%c[%3.3f]",c,x);
@@ -1461,42 +1485,6 @@ void font_get_parskip(double *ls,double *gap) {
 	*gap = *ls * .1;
 }
 
-int pass_font(const char *p) {
-	int j;
-	char u[90];
-	char vv[80],*s;
-	double xx;
-	s = &u[0];
-	strncpy(u,p,90);
-	if ( (*s=='"')  || (strchr(s,'$') != NULL)) {
-		strcpy(vv,"cvtfont(");
-		strcat(vv,s); strcat(vv,")");
-		polish_eval(vv,&xx);
-		memcpy(&j,&xx,sizeof(int));
-		return j;  /* actually is an int */
-	} else {
-		if (fnt.size() == 0) font_load();
-		for (unsigned int i = 1; i < fnt.size(); i++) {
-			if (fnt[i]->name != NULL && str_i_equals(fnt[i]->name, u)) {
-				return i;
-			}
-		}
-		ostringstream err;
-		err << "invalid font name: '" << u << "', expecting one of:";
-		for (unsigned int i = 1; i < fnt.size(); i++) {
-			if ((i-1) % 5 == 0) {
-				err << endl << "       ";
-			} else {
-				err << " ";
-			}
-			err << fnt[i]->name;
-		}
-		gprint(err.str().c_str());
-		/* default font number */
-		return 1;
-	}
-}
-
 #define get_exps(ss) polish(ss,(char *) pcode,plen,&etype)
 #define tok(n)  tk[n]
 
@@ -1830,10 +1818,7 @@ void fftext_block(const string& s,double width,int justify) {
 }
 
 void cmd_token(uchar **in,char *cmdstr) {
-	int i;
-	char *s;
-	s = cmdstr;
-	i=0;
+	int i = 0;
 	if ( (!isalpha(**in)) && (**in != 0)) {
 		if ((*in)[0] == '\'' && (*in)[1] == '\'') {
 			// special case for two subsequent single quotes
