@@ -55,10 +55,13 @@
 #include "var.h"
 #include "keyword.h"
 #include "run.h"
+#include "gle-poppler.h"
 
 #define BEGINDEF extern
 #include "begin.h"
 
+#include <time.h>
+
 void decode_utf8_basic(string& sc);
 
 extern ConfigCollection g_Config;
@@ -203,7 +206,7 @@ TeXObject* TeXInterface::drawObj(TeXHashObject* hobj, TeXObjectInfo& info, GLERe
 		/* Apply input transformation */
 		double devx, devy;
 		g_dev(xp, yp, &devx, &devy);
-		obj->setDeviceXY(devx/72.0*CM_PER_INCH, devy/72.0*CM_PER_INCH);
+		obj->setDeviceXY(devx/PS_POINTS_PER_INCH*CM_PER_INCH, devy/PS_POINTS_PER_INCH*CM_PER_INCH);
 		/* Find out if text should be rotated */
 		double angle = g_get_angle_deg();
 		if (fabs(angle) > 1e-6) {
@@ -217,11 +220,11 @@ TeXObject* TeXInterface::drawObj(TeXHashObject* hobj, TeXObjectInfo& info, GLERe
 }
 
 void TeXInterface::checkObjectDimensions() {
-	PSGLEDevice* psdev = (PSGLEDevice*)g_get_device_ptr();
+	GLEDevice* psdev = g_get_device_ptr();
 	double x0 = 0.0;
 	double y0 = 0.0;
-	double x1 = (double)psdev->getBoundingBox()->getX()/72.0*CM_PER_INCH;
-	double y1 = (double)psdev->getBoundingBox()->getY()/72.0*CM_PER_INCH;
+	double x1 = (double)psdev->getBoundingBox()->getX()/PS_POINTS_PER_INCH*CM_PER_INCH;
+	double y1 = (double)psdev->getBoundingBox()->getY()/PS_POINTS_PER_INCH*CM_PER_INCH;
 	for (vector<TeXObject*>::size_type i = 0; i < m_TeXObjects.size(); i++) {
 		TeXObject* obj = m_TeXObjects[i];
 		TeXHashObject* hobj = obj->getObject();
@@ -913,9 +916,8 @@ void TeXObject::output(ostream& os) {
 	}
 	os << "\\makebox(0,0)[lb]{";
 	if (!isBlack()) {
-		rgb01 c1;
-		g_colortyp_to_rgb01(getColor(), &c1);
-		os << "\\color[rgb]{" << c1.red << "," << c1.green << "," << c1.blue << "}";
+		GLERC<GLEColor> color(getColor());
+		os << "\\color[rgb]{" << color->getRed() << "," << color->getGreen() << "," << color->getBlue() << "}";
 	}
 	getObject()->outputLines(os);
 	for (int i = 0; i < closeb; i++) {
@@ -932,7 +934,8 @@ void TeXObject::getDimensions(double* x1, double *y1, double *x2, double *y2) {
 }
 
 int TeXObject::isBlack() {
-	return g_is_black(getColor());
+	GLEColor* color = getColor();
+	return color == 0 || color->getHexValueGLE() == GLE_COLOR_BLACK;
 }
 
 TeXHashObject::TeXHashObject(const string& line) : m_Line(line) {
@@ -1006,8 +1009,8 @@ void TeXObjectInfo::setJustify(int just) {
 	m_Status |= TEX_OBJ_INF_HAS_JUSTIFY;
 }
 
-void TeXObjectInfo::setColor(colortyp* color) {
-	m_Color = *color;
+void TeXObjectInfo::setColor(const GLERC<GLEColor>& color) {
+	m_Color = color;
 	m_Status |= TEX_OBJ_INF_HAS_COLOR;
 }
 
@@ -1024,7 +1027,7 @@ void TeXObjectInfo::initializeAll() {
 		g_get_just(&m_Just);
 	}
 	if ((m_Status & TEX_OBJ_INF_HAS_COLOR) == 0) {
-		g_get_colortyp(&m_Color);
+		m_Color = g_get_color();
 	}
 }
 
@@ -1054,6 +1057,7 @@ bool read_eps_and_adjust_bounding_box(const string& name, GLEScript* script) {
 			int b4New = (int)ceil(b2 + bb.getY() + 1e-6);
 			out << "%%BoundingBox: " << b1 << " " << b2 << " " << b3New << " " << b4New << endl;
 			script->setBoundingBoxOrigin(b1, b2);
+			script->setBoundingBox(b3New - b1 + 1, b4New - b2 + 1);
 		}
 		else if (str_starts_with_trim(line, "%%HiResBoundingBox") != -1 ||
 		         str_starts_with_trim(line, "%%Creator") != -1 ||
@@ -1081,20 +1085,24 @@ bool read_eps_and_adjust_bounding_box(const string& name, GLEScript* script) {
 	return true;
 }
 
-bool create_bitmap_file(GLEFileLocation* fname, int device, int dpi, bool bw, bool transp, GLEScript* script) {
+bool create_bitmap_file_ghostscript(GLEFileLocation* fname, int device, int dpi, int options, GLEScript* script) {
 	ostringstream gsargs;
 	gsargs << "-q -DNOPLATFONTS -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -dBATCH -dNOPAUSE -r";
 	gsargs << dpi;
-	GLEPoint bb(script->getBoundingBox());
-	GLEPoint origin(script->getBoundingBoxOrigin());
-	int img_wd = GLEBBoxToPixels(dpi, bb.getX());
-	int img_hi = GLEBBoxToPixels(dpi, bb.getY());
-	gsargs << " -g" << img_wd << "x" << img_hi;
+	string* bytesPDF = script->getRecordedBytesBuffer(GLE_DEVICE_PDF);
+	if (bytesPDF->empty()) {
+		GLEPoint bb(script->getBoundingBox());
+		int img_wd = GLEBBoxToPixels(dpi, bb.getX());
+		int img_hi = GLEBBoxToPixels(dpi, bb.getY());
+		gsargs << " -g" << img_wd << "x" << img_hi;
+	}
 	string gs_opts = g_CmdLine.getOptionString(GLE_OPT_GSOPTIONS);
 	if (gs_opts != "") {
 		str_replace_all(gs_opts, "\\", "");
 		gsargs << " " << gs_opts;
 	}
+	bool bw = (options & GLE_OUTPUT_OPTION_GRAYSCALE) != 0;
+	bool transp = (options & GLE_OUTPUT_OPTION_TRANSPARENT) != 0;
 	gsargs << " -sDEVICE=";
 	switch (device) {
 		case GLE_DEVICE_PNG:
@@ -1120,11 +1128,35 @@ bool create_bitmap_file(GLEFileLocation* fname, int device, int dpi, bool bw, bo
 		gsargs << " -sOutputFile=\"" << outputfile << "\"";
 	}
 	gsargs << " -";
-	stringstream postscript;
-	string* bytes = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
-	postscript << (-origin.getX()) << " " << (-origin.getY()) << " translate" << endl;
-	postscript.write(bytes->data(), bytes->size());
-	return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), &postscript);
+	string* bytesEPS = script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
+	if (!bytesPDF->empty()) {
+		stringstream pdfCode;
+		pdfCode.write(bytesPDF->data(), bytesPDF->size());
+		return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), &pdfCode);
+	} else {
+		stringstream postscript;
+		GLEPoint origin(script->getBoundingBoxOrigin());
+		postscript << (-origin.getX()) << " " << (-origin.getY()) << " translate" << endl;
+		postscript.write(bytesEPS->data(), bytesEPS->size());
+		return run_ghostscript(gsargs.str(), outputfile, !fname->isStdout(), &postscript);
+	}
+}
+
+bool create_bitmap_file(GLEFileLocation* fname, int device, int dpi, int options, GLEScript* script) {
+#ifdef HAVE_POPPLER
+	bool supportsBitmapType = g_bitmap_supports_type(g_device_to_bitmap_type(device));
+	string* bytesPDF = script->getRecordedBytesBuffer(GLE_DEVICE_PDF);
+	if (supportsBitmapType && !bytesPDF->empty()) {
+		std::string myFName = fname->getFullPath();
+		myFName += g_device_to_ext(device);
+		if (g_verbosity() >= 5) {
+			g_message(std::string("[Poppler PDF conversion: ") + myFName + "]");
+		}
+		gle_convert_pdf_to_image_file((char*)bytesPDF->c_str(), (int)bytesPDF->size(), dpi, device, options, myFName.c_str());
+		return true;
+	}
+#endif // HAVE_POPPLER
+	return create_bitmap_file_ghostscript(fname, device, dpi, options, script);
 }
 
 bool create_pdf_file_ghostscript(GLEFileLocation* fname, int dpi, GLEScript* script) {
@@ -1170,6 +1202,8 @@ bool create_pdf_file_ghostscript(GLEFileLocation* fname, int dpi, GLEScript* scr
 	int img_hi = GLEBBoxToPixels(dpi, bb.getY());
 	gsargs << " -g" << img_wd << "x" << img_hi;
 	gsargs << " -sDEVICE=pdfwrite";
+	gsargs << " -dPDFSETTINGS=/prepress -dMaxSubsetPct=100 -dSubsetFonts=true";
+	gsargs << " -dEmbedAllFonts=true -dAutoRotatePages=/None";
 	string outputfile;
 	if (fname->isStdout()) {
 		gsargs << " -sOutputFile=-";
@@ -1256,7 +1290,7 @@ bool post_run_latex(bool result_ok, stringstream& output, const string& cmdline)
 	}
 }
 
-bool create_pdf_file_pdflatex(const string& fname) {
+bool create_pdf_file_pdflatex(const string& fname, GLEScript* script) {
 	string file, dir;
 	SplitFileName(fname, dir, file);
 	ConfigSection* tools = g_Config.getSection(GLE_CONFIG_TOOLS);
@@ -1281,6 +1315,13 @@ bool create_pdf_file_pdflatex(const string& fname) {
 	post_run_latex(result_ok, output, cmdline);
 	DeleteFileWithExt(fname, ".aux");
 	DeleteFileWithExt(fname, ".log");
+	if (result_ok) {
+		std::vector<char> pdfData;
+		if (GLEReadFileBinary(outfile, &pdfData) && !pdfData.empty()) {
+			string* buffer = script->getRecordedBytesBuffer(GLE_DEVICE_PDF);
+			*buffer = std::string(&pdfData[0], pdfData.size());
+		}
+	}
 	return result_ok;
 }
 
