@@ -313,6 +313,40 @@ void g_throw_parser_error(const char* err, int idx) throw(ParserError) {
 	throw err_exp;
 }
 
+IThrowsError::IThrowsError() {
+}
+
+IThrowsError::~IThrowsError() {
+}
+ 
+ParserError IThrowsError::throwError(const char* s1, const char* s2, const char* s3) {
+	TokenizerPos pos;
+	pos.setColumn(-1);
+	string err(s1);
+	if (s2 != NULL) err += s2;
+	if (s3 != NULL) err += s3;
+	return ParserError(err, pos, NULL);
+}
+ 
+ParserError IThrowsError::throwError(int /* pos */, const string& error) {
+   return throwError(error);
+}
+
+ParserError IThrowsError::throwError(const string& error) {
+   TokenizerPos pos;
+	pos.setColumn(-1);
+	return ParserError(error, pos, NULL);
+}
+
+int IThrowsError::getErrorPosition() const {
+   return 0;
+}
+
+IThrowsError* g_get_throws_error() {
+   static IThrowsError instance;
+   return &instance;
+}
+
 ParserError::ParserError(const string& txt, const TokenizerPos& pos, const char* fname) {
 	m_txt = txt;
 	str_replace_all(m_txt, "\n", "\n>> ");
@@ -701,18 +735,18 @@ void Tokenizer::get_token_2() throw(ParserError) {
 	}
 	// String with \" or \' marks
 	if ((token_ch  == '\"' || token_ch == '\'') && m_language->getParseStrings()) {
-      char string_delim = token_ch;
+		char string_delim = token_ch;
 		m_token = token_ch;
 		do {
 			token_ch = token_read_char_no_comment();
-         m_token += token_ch;
+			m_token += token_ch;
 			if (token_ch == string_delim) {
 				token_ch = token_read_char_no_comment();
 				if (token_ch != string_delim) {
 					token_pushback_ch(token_ch);
 					return;
 				}
-         }
+			}
 		} while (m_token_at_end == 0);
 		throw error("unterminated string constant");
 	}
@@ -761,16 +795,18 @@ void Tokenizer::get_token_2() throw(ParserError) {
 	}
 }
 
-void Tokenizer::copy_string(char endch) throw(ParserError) {
-	int prev_backslash = 0;
+void Tokenizer::copy_string(char string_delim) throw(ParserError) {
 	TokenizerPos pos = token_stream_pos();
 	while (m_token_at_end == 0) {
 		char token_ch = token_read_char_no_comment();
 		m_token += token_ch;
-		// Return if at '"' and not preceded by odd number of backslashes
-		if (token_ch == endch && (prev_backslash % 2 == 0)) return;
-		if (token_ch == '\\') prev_backslash++;
-		else prev_backslash = 0;
+		if (token_ch == string_delim) {
+			token_ch = token_read_char_no_comment();
+			if (token_ch != string_delim) {
+				token_pushback_ch(token_ch);
+				return;
+			}
+		}
 	}
 	throw error(pos, "unterminated string constant");
 }
@@ -1255,6 +1291,22 @@ void Tokenizer::read_till_close_comment() throw(ParserError) {
 	}
 }
 
+ParserError Tokenizer::throwError(const char* s1, const char* s2, const char* s3) {
+   return error(s1, s2, s3);
+}
+
+ParserError Tokenizer::throwError(int pos, const string& msg) {
+   return error(pos, msg);
+}
+   
+ParserError Tokenizer::throwError(const string& msg) {
+   return error(msg);
+}
+   
+int Tokenizer::getErrorPosition() const {
+   return token_pos_col();
+}
+
 ParserError Tokenizer::error(const char* s1, const char* s2, const char* s3) const {
    ostringstream err;
    err << s1 << s2 << s3;
