@@ -38,6 +38,8 @@
 #include "consolewindow.h"
 #include "dialogues.h"
 #include "3dviewer.h"
+#include "../gle/cutils.h"
+#include "../gle/gle-block.h"
 #include "../gle/surface/gsurface.h"
 // #include "downloader.h"
 
@@ -72,8 +74,8 @@ GLEMainWindow::GLEMainWindow(int argc, char *argv[])
 
 	// Initialise temp file names
 	// Should be unique names - otherwise can't run multiple QGLE instances
-	tempFiles[PreviewModeEPS] = createTempFile(tr(".eps"));
-	tempFiles[EditModeEPS] = createTempFile(tr(".eps"));
+	tempFiles[PreviewModeOutput] = createTempFile(tr(".out"));
+	tempFiles[EditModeOutput] = createTempFile(tr(".out"));
 	tempFiles[NewGLE] = createTempFile(tr(".gle"));
 
 	// Create the drawing area in a scroll widget.  For high resolutions,
@@ -89,7 +91,7 @@ GLEMainWindow::GLEMainWindow(int argc, char *argv[])
 	splitWindow->setStretchFactor(0, 1);
 	splitWindow->setStretchFactor(1, 0);
 	gleInterface->setOutputStream(consoleWindow->getOutput());
-	gleInterface->initializeGLE("qgle", argv);
+	gleInterface->initializeGLE("qgle", argc, argv);
 
 	QList<int> splitSizes;
 	splitSizes.append(50);
@@ -318,12 +320,10 @@ void GLEMainWindow::checkForFileUpdates()
 		switch(answer)
 		{
 			case 0:
-				qDebug() << "Reload file selected";
 				openFile(currFile->primaryFile());
 				isOpen = true;
 				break;
 			default:
-				qDebug() << "Ignore selected";
 				break;
 		}
 
@@ -357,9 +357,6 @@ void GLEMainWindow::dropEvent(QDropEvent *event)
 
 	if (rx.exactMatch(uri))
 	{
-		qDebug() << "Dropped file: " <<
-			uri << ": " << rx.capturedTexts()[2];
-		QGLE::flushIO();
 		openFile(rx.capturedTexts()[2]);
 	}
 	else
@@ -477,22 +474,17 @@ void GLEMainWindow::keyPressEvent(QKeyEvent *event)
 			break;
 
 		case Qt::Key_Shift:
-			qDebug() << "Shift key depressed";
 			drawingArea->modifierToggle(GLEDrawingArea::ShiftKey, true);
 			break;
 
 		case Qt::Key_Control:
-			qDebug() << "Control key depressed";
 			drawingArea->modifierToggle(GLEDrawingArea::CtrlKey, true);
 			break;
 
 		case Qt::Key_Alt:
-			qDebug() << "Alt key depressed";
 			drawingArea->modifierToggle(GLEDrawingArea::AltKey, true);
 			break;
 
-
-
 		default:
 			event->ignore();
 	}
@@ -510,17 +502,14 @@ void GLEMainWindow::keyReleaseEvent(QKeyEvent *event)
 	switch (event->key())
 	{
 		case Qt::Key_Shift:
-			qDebug() << "Shift key released";
 			drawingArea->modifierToggle(GLEDrawingArea::ShiftKey, false);
 			break;
 
 		case Qt::Key_Control:
-			qDebug() << "Control key released";
 			drawingArea->modifierToggle(GLEDrawingArea::CtrlKey, false);
 			break;
 
 		case Qt::Key_Alt:
-			qDebug() << "Alt key released";
 			drawingArea->modifierToggle(GLEDrawingArea::AltKey, false);
 			break;
 
@@ -564,7 +553,6 @@ void GLEMainWindow::initRenderThread()
 // Start the server
 void GLEMainWindow::startServer()
 {
-	qDebug() << "Starting server";
 	// Initialise the GS library
 	initLibGS();
 
@@ -1316,8 +1304,8 @@ GLEFileInfo* GLEMainWindow::getNonCurrentModeFileInfo()
 //! Set temp files correctly for GLE file
 void GLEMainWindow::initTempFilesGLE()
 {
-	currentFile[GLESettings::PreviewMode].setEpsFile(tempFiles[PreviewModeEPS]);
-	currentFile[GLESettings::EditMode].setEpsFile(tempFiles[EditModeEPS]);
+	currentFile[GLESettings::PreviewMode].setEpsFile(tempFiles[PreviewModeOutput]);
+	currentFile[GLESettings::EditMode].setEpsFile(tempFiles[EditModeOutput]);
 }
 
 //! Update currentFile on change PreviewMode <-> EditMode
@@ -1357,7 +1345,6 @@ void GLEMainWindow::previewModeToggle(bool state)
 		// or if objects have been moved
 		if (drawingArea->thereAreNewObjects())
 		{
-			qDebug() << "Rendering new objects";
 			updateGLE();
 			if (settings->saveOnPreview())
 			{
@@ -1549,27 +1536,27 @@ void GLEMainWindow::renderEPS(double dpi)
 }
 
 // Render an EPS file from a given QString
-void GLEMainWindow::renderEPS(QString epsFile, double dpi, const QSize& area)
+void GLEMainWindow::renderEPS(QString outputFile, double dpi, const QSize& area)
 {
-	qDebug() << "About to render " << epsFile;
 	if (getCurrentFile()->isGLE())
 	{
 		GLEScript* script = getGLEScript();
-		if (!script->getRecordedBytesBuffer(GLE_DEVICE_EPS)->empty())
+		if (!script->getRecordedBytesBuffer(GLE_DEVICE_EPS)->empty()
+			|| !script->getRecordedBytesBuffer(GLE_DEVICE_PDF)->empty())
 		{
 			updateEnableActions(false);
-			renderThread->renderEPSFromMemoryToImage(script, epsFile, dpi, area);
+			renderThread->renderOutputFromMemoryToImage(script, outputFile, dpi, area);
 			return;
 		}
 	}
 	else
 	{
-		QFileInfo fi(epsFile);
+		QFileInfo fi(outputFile);
 		// If the file is readable, re-render it in the new scale
 		if (fi.isReadable())
 		{
 			updateEnableActions(false);
-			renderThread->renderEPSFromFileToImage(epsFile, dpi, area);
+			renderThread->renderEPSFromFileToImage(outputFile, dpi, area);
 			return;
 		}
 	}
@@ -1698,6 +1685,9 @@ void GLEMainWindow::renderGLE(double dpi, const QSize& area)
 	} else {
 		gleInterface->setCompatibilityMode("4.2");
 	}
+	if (settings->isRenderUsingCairo()) {
+		gleInterface->setCmdLineOption("cairo");
+	}
 	updateEnableActions(false);
 	todoList.append(ToDoAfterRenderGLE);
 	renderThread->renderGLEToImage(getGLEScript(), getCurrentFile()->epsFile(), dpi, area);
@@ -2148,7 +2138,6 @@ void GLEMainWindow::browseScriptFolder()
 // SLOT: Open an existing file
 void GLEMainWindow::openFile(QString fileName, bool isOnStartup)
 {
-	qDebug() << "Open file " << fileName;
 	bool isReload = (getCurrentGleFile() == fileName);
 
 	// If changes have been made, ask what to do
@@ -2195,7 +2184,6 @@ void GLEMainWindow::openFile(QString fileName, bool isOnStartup)
 		// Get the file information
 		QFileInfo fname(fileName);
 		settings->setPwd(fname.dir().path());
-		qDebug() << "Open file: " << fileName;
 		// Regular expressions to check whether it's an EPS file or a GLE file
 		QRegExp rxeps(".*\\.eps$", Qt::CaseInsensitive);
 		QRegExp rxgle(".*\\.gle$", Qt::CaseInsensitive);
@@ -2581,7 +2569,6 @@ void GLEMainWindow::openInTextEditor(const QString& file)
 #ifdef Q_WS_WIN
 		filename.replace('/', '\\');
 #endif
-		qDebug() << "Opening:" << filename;
 
 #ifdef Q_WS_MAC
 		// This checks whether we're on a Mac and if so, handles .app files appropriately
@@ -2647,7 +2634,7 @@ void GLEMainWindow::renderPostscript(const char* ps, const GLERectangle& rect, d
 {
 	// qDebug() << "Rendering: " << rect.getWidth() << " x " << rect.getHeight();
 	renderThread->startRender(rect, dpi);
-	QString translate = QString("%1 %2 translate\n").arg(-rect.getXMin()*72.0/CM_PER_INCH).arg(-rect.getYMin()*72.0/CM_PER_INCH);
+	QString translate = QString("%1 %2 translate\n").arg(-rect.getXMin()*PS_POINTS_PER_INCH/CM_PER_INCH).arg(-rect.getYMin()*PS_POINTS_PER_INCH/CM_PER_INCH);
 	renderThread->nextRender(translate.toLatin1().constData());
 	renderThread->nextRender(gleInterface->getInitialPostScript());
 	renderThread->nextRender(ps);
